﻿Public Class Controller

#Region "Keyboard Hook API Declarations"

    Structure KBDLLHOOKSTRUCT
        Public vkCode As Integer
        Public scanCode As Integer
        Public flags As Integer
        Public time As Integer
        Public dwExtraInfo As Integer
    End Structure

    Private Declare Function UnhookWindowsHookEx Lib "user32" ( _
        ByVal hHook As Integer) As Integer
    Private Declare Function SetWindowsHookEx Lib "user32" Alias "SetWindowsHookExA" ( _
        ByVal idHook As Integer, _
        ByVal lpfn As KeyboardHookDelegate, _
        ByVal hmod As Integer, _
        ByVal dwThreadId As Integer) As Integer
    Private Declare Function CallNextHookEx Lib "user32" ( _
        ByVal hHook As Integer, _
        ByVal nCode As Integer, _
        ByVal wParam As Integer, _
        ByVal lParam As KBDLLHOOKSTRUCT) As Integer
    Private Delegate Function KeyboardHookDelegate( _
        ByVal Code As Integer, _
        ByVal wParam As Integer, _
        ByRef lParam As KBDLLHOOKSTRUCT) As Integer


    'Konstanten
    Private Const WM_KEYUP As Integer = &H101
    Private Const WM_SYSKEYUP As Long = &H105
    Private Const WH_KEYBOARD_LL As Long = 13

    Private Const VK_RCONTROL As Long = &HA3
    Private Const VK_UP As Long = &H26
    Private Const VK_DOWN As Long = &H28
    Private Const VK_LEFT As Long = &H25
    Private Const VK_RIGHT As Long = &H27

#End Region

    Public Shared inst As Controller

    Private Enum PlayerStatus
        Playing
        Pause
        [Stop]
    End Enum


    'Callbacks
    Dim cbKeyboard As KeyboardHookDelegate = Nothing

    'Handles
    Dim hwndKeyboard As IntPtr = 0

    'Fenster
    Dim charmapWindow As Charmap = Nothing
    Dim playerWindow As Player = Nothing
    Dim movieWindow As Movie

    'Player
    Dim playerControl As MediaElement
    Dim status As PlayerStatus = PlayerStatus.Stop

    'Library
    Dim libraryWindow As Library
    Dim mlib As Media
    Dim mplaylist As Playlist

    ReadOnly Property Library As Media
        Get
            Return mlib
        End Get
    End Property


    Sub New(ByVal player As Player)
        Try
            inst = Me
            mlib = New Media
            mplaylist = Playlist.fromFile(LastPlayedList)

            playerWindow = player
            HookKeyboard()

            movieWindow = New Movie
            playerControl = movieWindow.mediaPlayer

            libraryWindow = New Library

            'Dim fs = New IO.FileStream("c:\ttt", IO.FileMode.Open)
        Catch ex As IO.FileNotFoundException
            Clipboard.Clear()
            Clipboard.SetText(ex.Message & vbCrLf & ex.StackTrace())
            MessageBox.Show(ex.Message & vbCrLf & ex.StackTrace())
        End Try

    End Sub

    Sub addMediaFileToPlaylist(ByVal media As Media.mediaElement)
        mplaylist.addFile(media)
    End Sub

    Sub unload()
        closeCharmap()
        UnhookKeyboard()
        movieWindow.Close()
        libraryWindow.Close()

        mplaylist.savePlaylistM3U(LastPlayedList)
    End Sub

    ReadOnly Property LastPlayedList As String
        Get
            Dim path() As String = { _
                Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), _
                "Playr", "Library", "lastPlayed.m3u"}
            Return String.Join(IO.Path.DirectorySeparatorChar, path)
        End Get
    End Property

    Sub switchLibrary()
        If libraryWindow.IsVisible Then
            libraryWindow.Hide()
        Else
            libraryWindow.Show()
        End If

    End Sub

    Sub closeCharmap()
        If isCharmapVisible() Then
            charmapWindow.Close()
            charmapWindow = Nothing
        End If
    End Sub

    Sub showCharmap()
        If isCharmapVisible() Then
            Return
        End If

        charmapWindow = New Charmap
        charmapWindow.Show()
    End Sub

    Function isCharmapVisible() As Boolean
        If charmapWindow Is Nothing Then
            Return False
        End If

        Return True
    End Function

    Property Playlist As Playlist
        Get
            Return mplaylist
        End Get
        Set(ByVal value As Playlist)
            mplaylist = value
            updatePlaylistSource()
        End Set
    End Property

    Sub updatePlaylistSource()
        libraryWindow.updatePlaylistSource()
    End Sub

    Sub addDirectory(ByVal dir As String)
        Dim media As Media.mediaElement

        For Each d As String In IO.Directory.GetDirectories(dir)
            addDirectory(d)
        Next

        For Each f As String In IO.Directory.GetFiles(dir)
            media = mlib.addMediaElement(f)
            mplaylist.addFile(media)
        Next
    End Sub

    Sub DropFiles(ByVal e As System.Windows.DragEventArgs)
        Dim files = e.Data.GetData(DataFormats.FileDrop, True)
        Dim media As Media.mediaElement
        Dim exts = ".mp3;.avi;.mpg;.mpeg;.wav,.mid;.midi;.wav;.wma;.wmv".Split(";")

        For Each filename As String In files
            If (New IO.DirectoryInfo(filename)).Attributes = IO.FileAttributes.Directory Then
                addDirectory(filename)
                Continue For
            End If

            If exts.Contains(System.IO.Path.GetExtension(filename)) Then
                media = mlib.addMediaElement(filename)
                mplaylist.addFile(media)
            End If
        Next

        Play()
    End Sub

#Region "Player Control"
    Sub Play()
        If status <> PlayerStatus.Playing Then
            If playerControl.Source Is Nothing Or Not mplaylist.isPlaying Then
                If mplaylist.Length = 0 Then
                    Return
                End If

                playerControl.Source = New Uri(mplaylist.getNext().Filename)
            End If

            playerControl.Play()
            status = PlayerStatus.Playing

            playerWindow.changePlayStatus(isPlaying)
            playerWindow.changeTitle(mplaylist.current.Title)
        End If
    End Sub

    Sub PlayFile(ByVal media As Media.mediaElement)
        playerControl.Source = New Uri(mplaylist.getNext(media).Filename)
        playerControl.Play()
        status = PlayerStatus.Playing

        playerWindow.changePlayStatus(isPlaying)
        playerWindow.changeTitle(mplaylist.current.Title)
    End Sub

    Sub [Stop]()
        If status <> PlayerStatus.Stop Then
            playerControl.Stop()
            status = PlayerStatus.Stop
        End If
    End Sub

    Sub Pause()
        If status <> PlayerStatus.Pause Then
            playerControl.Pause()
            status = PlayerStatus.Pause
            playerWindow.changePlayStatus(isPlaying)
        End If
    End Sub

    Sub PlayForward()
        If mplaylist.Length = 0 Then
            [Stop]()
            Return
        End If

        playerControl.Source = New Uri(mplaylist.getNext().Filename)
        playerWindow.changeTitle(mplaylist.current.Title)
    End Sub

    Sub PlayBackward()
        playerControl.Source = New Uri(mplaylist.getPrev().Filename)
        playerWindow.changeTitle(mplaylist.current.Title)
    End Sub

    ReadOnly Property isPlaying As Boolean
        Get
            Return status = PlayerStatus.Playing
        End Get
    End Property

    Property Position() As Double
        Get
            Return playerControl.Position.TotalMilliseconds
        End Get
        Set(ByVal value As Double)
            playerControl.Position = New TimeSpan(0, 0, 0, 0, value)
        End Set
    End Property

    ReadOnly Property Maximum() As Double
        Get
            Return playerControl.NaturalDuration.TimeSpan.TotalMilliseconds
        End Get
    End Property
#End Region

#Region "Movie Window Communication"
    Public Sub onMediaOpened(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs)
        playerWindow.onMediaOpened(sender, e)
        playerWindow.tbContent.Text = mplaylist.current.Filename + " - Dauer:" + playerControl.NaturalDuration.ToString()
    End Sub

    Public Sub onMediaEnded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs)
        PlayForward()
    End Sub

#End Region

    ''' <summary>
    ''' Get an shared object of the controller class
    ''' </summary>
    ''' <param name="ref">must passed only the first time this function is called</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function instance(Optional ByVal ref As Player = Nothing) As Controller
        If ref IsNot Nothing Then
            inst = New Controller(ref)
        End If

        Return inst
    End Function



#Region "Keyboard Hook"
    Public Function KeyboardCallback(ByVal Code As Integer, ByVal wParam As Integer, ByRef lParam As KBDLLHOOKSTRUCT) As Integer
        If wParam = WM_KEYUP Or wParam = WM_SYSKEYUP Then
            If lParam.vkCode = VK_RCONTROL Then
                If isCharmapVisible() Then
                    closeCharmap()
                Else
                    showCharmap()
                End If
            ElseIf isCharmapVisible() Then
                Select Case lParam.vkCode
                    Case VK_UP
                        Pause()

                    Case VK_DOWN
                        Play()

                    Case VK_LEFT
                        PlayBackward()

                    Case VK_RIGHT
                        PlayForward()

                End Select
            End If
        End If

        Return CallNextHookEx(hwndKeyboard, Code, wParam, lParam)
    End Function

    Public Sub UnhookKeyboard()
        If UnhookWindowsHookEx(hwndKeyboard) <> 0 Then
            hwndKeyboard = 0
        End If
    End Sub

    Public Sub HookKeyboard()
        cbKeyboard = New KeyboardHookDelegate(AddressOf KeyboardCallback)
        hwndKeyboard = SetWindowsHookEx(WH_KEYBOARD_LL, cbKeyboard, Process.GetCurrentProcess.MainModule.BaseAddress, 0)
    End Sub
#End Region

End Class
