﻿Imports System.IO

Public Class Encoding

    Shared Function detectFile(ByVal file As Stream) As StreamReader
        Dim data(4) As Byte
        Dim enc = System.Text.Encoding.Default

        If file.Length < 4 Then
            Return New StreamReader(file, enc)
        End If

        file.Read(data, 0, 4)
        file.Position = 0

        'http://de.wikipedia.org/wiki/Byte_Order_Mark
        If data.Length > 3 AndAlso (data(0) = &HEF And data(1) = &HBB And data(2) = &HBF) Then
            'UTF-8
            enc = System.Text.Encoding.UTF8
        ElseIf data.Length > 3 AndAlso (data(0) = &HFF And data(1) = &HFE And data(2) = 0 And data(3) = 0) Then
            enc = System.Text.Encoding.UTF32
        ElseIf data(0) = &HFF And data(1) = &HFE Then
            'UTF-16LE
            enc = System.Text.Encoding.Unicode
        ElseIf (data(0) = &HFE And data(1) = &HFF) Or _
            (data(0) = 0 And data(1) = 0) Then
            'UTF-16BE bzw. UTF-32BE
            Throw New NotSupportedException()
        End If

        'Standard Kodierung - wir nehmen einfach mal Unicode an...
        Return New StreamReader(file, enc)
    End Function


    'The AutoDetect function is from SharpDevelop(Libraries\ICSharpCode.TextEditor\Project\Src\Util\FileReader.cs)
    Private Shared Function AutoDetect(ByVal fs As Stream, ByVal firstByte As Byte, ByVal secondByte As Byte, ByVal defaultEncoding As System.Text.Encoding) As StreamReader
        Dim max As Integer = CInt(Math.Min(fs.Length, 500000))
        ' look at max. 500 KB
        Const ASCII As Integer = 0
        Const [Error] As Integer = 1
        Const UTF8 As Integer = 2
        Const UTF8Sequence As Integer = 3
        Dim state As Integer = ASCII
        Dim sequenceLength As Integer = 0
        Dim b As Byte

        For i As Integer = 0 To max - 1
            If i = 0 Then
                b = firstByte
            ElseIf i = 1 Then
                b = secondByte
            Else
                b = CByte(fs.ReadByte())
            End If
            If b < &H80 Then
                ' normal ASCII character
                If state = UTF8Sequence Then
                    state = [Error]
                    Exit For
                End If
            ElseIf b < &HC0 Then
                ' 10xxxxxx : continues UTF8 byte sequence
                If state = UTF8Sequence Then
                    sequenceLength -= 1
                    If sequenceLength < 0 Then
                        state = [Error]
                        Exit For
                    ElseIf sequenceLength = 0 Then
                        state = UTF8
                    End If
                Else
                    state = [Error]
                    Exit For
                End If
            ElseIf b >= &HC2 AndAlso b < &HF5 Then
                ' beginning of byte sequence
                If state = UTF8 OrElse state = ASCII Then
                    state = UTF8Sequence
                    If b < &HE0 Then
                        ' one more byte following
                        sequenceLength = 1
                    ElseIf b < &HF0 Then
                        ' two more bytes following
                        sequenceLength = 2
                    Else
                        ' three more bytes following
                        sequenceLength = 3
                    End If
                Else
                    state = [Error]
                    Exit For
                End If
            Else
                ' 0xc0, 0xc1, 0xf5 to 0xff are invalid in UTF-8 (see RFC 3629)
                state = [Error]
                Exit For
            End If
        Next
        fs.Position = 0
        Select Case state
            Case ASCII, [Error]
                ' when the file seems to be ASCII or non-UTF8,
                ' we read it using the user-specified encoding so it is saved again
                ' using that encoding.
                If IsUnicode(defaultEncoding) Then
                    ' the file is not Unicode, so don't read it using Unicode even if the
                    ' user has choosen Unicode as the default encoding.

                    ' If we don't do this, SD will end up always adding a Byte Order Mark
                    ' to ASCII files.
                    ' use system encoding instead
                    defaultEncoding = System.Text.Encoding.Default
                End If
                Return New StreamReader(fs, defaultEncoding)
            Case Else
                Return New StreamReader(fs)
        End Select
    End Function

    Public Shared Function IsUnicode(ByVal encoding As System.Text.Encoding) As Boolean
        Dim codepage = encoding.CodePage
        'return true if codepage is any UTF codepage
        Return codepage = 65001 Or codepage = 65000 Or codepage = 1200 Or codepage = 1201
    End Function


End Class
