﻿Imports System.Linq
Imports System.Data
Imports System.Data.SqlServerCe
Imports System.IO
Imports System.Threading

Public Class Media

    Dim cn As SqlCeConnection
    Dim items As New Dictionary(Of String, mediaElement)

    Class mediaElement
        Private sFilename As String
        Private sTitle As String
        Private iLength As Integer
        Private aArtist As String
        Private aAlbum As String

        Private mNextMedia As Media.mediaElement
        Private mPrevMedia As Media.mediaElement

        Property nextMedia() As Media.mediaElement
            Get
                Return mNextMedia
            End Get
            Set(ByVal value As Media.mediaElement)
                mNextMedia = value
            End Set
        End Property

        Property prevMedia() As Media.mediaElement
            Get
                Return mPrevMedia
            End Get
            Set(ByVal value As Media.mediaElement)
                mPrevMedia = value
            End Set
        End Property

        Public tags As List(Of ID3.Frame)

        ReadOnly Property src() As String
            Get
                Return sFilename
            End Get
        End Property

        ReadOnly Property Title As String
            Get
                Return sTitle
            End Get
        End Property

        ReadOnly Property Artist As String
            Get
                Return aArtist
            End Get
        End Property

        ReadOnly Property Album As String
            Get
                Return aAlbum
            End Get
        End Property

        ReadOnly Property Filename As String
            Get
                Return sFilename
            End Get
        End Property

        Sub New(ByVal filename As String, ByVal title As String, ByVal length As String, ByVal artist As String, ByVal album As String)
            sFilename = filename
            stitle = title
            ilength = length
            aAlbum = album
            aArtist = artist
        End Sub
    End Class

    Sub New()
        initDatabase()
        initLibrary()
    End Sub


    Function searchMediaFiles(ByVal tag As String) As List(Of mediaElement)
        Dim cmd As SqlCeCommand
        Dim rs As SqlCeResultSet
        Dim res As New List(Of mediaElement)

        cmd = New SqlCeCommand("SELECT DISTINCT media FROM id3tag WHERE value LIKE @p", cn)
        cmd.Parameters.Add("@p", String.Format("%{0}%", tag))
        rs = cmd.ExecuteResultSet(ResultSetOptions.Scrollable)

        If rs.HasRows Then
            While (rs.Read)
                res.Add(items.Item(rs.Item("media")))
            End While
        End If

        Return res
    End Function

    Sub scanDirectory(ByVal dir As String)
        Dim thread = New Thread(AddressOf scanWorker)
        thread.Start(dir)
    End Sub

    Sub scanWorker(ByVal dir As String)
        For Each f As String In Directory.GetFiles(dir)
            If Path.GetExtension(f) = ".mp3" Then
                addMediaElement(f)
            End If
        Next

        For Each d As String In Directory.GetDirectories(dir)
            scanWorker(d)
        Next
    End Sub


    Sub initLibrary()
        Dim cmd As SqlCeCommand
        Dim rs As SqlCeResultSet

        cmd = New SqlCeCommand("SELECT * FROM media", cn)
        rs = cmd.ExecuteResultSet(ResultSetOptions.Scrollable)

        If rs.HasRows Then
            While (rs.Read)
                Me.items.Add(rs.Item("filename"), New mediaElement(rs.Item("filename"), rs.Item("title"), rs.Item("length"), rs.Item("author"), rs.Item("album")))
            End While
        End If
    End Sub

    Sub initDatabase()

        cn = New SqlCeConnection(connectionString)

        If Not File.Exists(LibraryFile) Then
            Dim en As New SqlServerCe.SqlCeEngine(connectionString)
            Dim cmd As SqlCeCommand

            'Pfad erstellen
            Directory.CreateDirectory(Path.GetDirectoryName(LibraryFile))
            'Datenbank erstellen
            en.CreateDatabase()

            'Verbindung öffnen
            cn.Open()

            'Tabellen erstellen..
            cmd = New SqlCeCommand("CREATE TABLE media ( " + _
                                   "filename NVARCHAR(255) NOT NULL PRIMARY KEY," + _
                                   "title NVARCHAR(200) NOT NULL," + _
                                   "length INT," + _
                                   "author NVARCHAR(100)," + _
                                   "album NVARCHAR(100)" + _
                                   ")", cn)
            cmd.ExecuteNonQuery()

            cmd = New SqlCeCommand("CREATE TABLE tag ( " + _
                                   "media NVARCHAR(255) NOT NULL," + _
                                   "text NVARCHAR(200) NOT NULL" + _
                                   ")", cn)

            cmd = New SqlCeCommand("CREATE TABLE id3tag ( " + _
                                   "media NVARCHAR(255) NOT NULL," + _
                                   "tag NVARCHAR(255) NOT NULL," + _
                                   "value NTEXT NOT NULL" + _
                                   ")", cn)

            cmd.ExecuteNonQuery()

            MessageBox.Show("Bei der ersten Programmausführung wird im Hintergrund der Musikordner durchsucht und indiziert")
            scanDirectory(Environment.GetFolderPath(Environment.SpecialFolder.MyMusic))
        Else
            cn.Open()
        End If

    End Sub

    Function addMediaElement(ByVal filename As String) As mediaElement
        'Relativen Pfad auflösen um Namen eindeutig zu machen
        filename = (New DirectoryInfo(filename)).FullName

        If items.ContainsKey(filename) Then
            Return items.Item(filename)
        End If

        Dim cmd As SqlCeCommand
        Dim elm As mediaElement
        Dim info = ID3.getID3V2Tag(filename)
        Dim artist As String = "Unknown"
        Dim album As String = ""
        Dim title As String = Path.GetFileNameWithoutExtension(filename)
        Dim exts = ".mp3;.avi;.mpg;.mpeg;.wav,.mid;.midi;.wav;.wma;.wmv".Split(";")

        If Not exts.Contains(Path.GetExtension(filename)) Then
            Return Nothing
        End If

        For Each tag As ID3.Frame In info
            cmd = New SqlCeCommand("INSERT INTO id3tag (media, tag, value) VALUES (@1, @2, @3)", cn)
            cmd.Parameters.Add("@1", filename)
            cmd.Parameters.Add("@2", tag.Name)
            cmd.Parameters.Add("@3", tag.Data)
            cmd.ExecuteNonQuery()

            Select Case tag.Name
                Case "TPE1", "TP1"
                    artist = tag.Data
                Case "TALB", "TAL"
                    album = tag.Data
                Case "TIT2", "TT2"
                    title = tag.Data
            End Select
        Next

        cmd = New SqlCeCommand("INSERT INTO media (filename, title, length, author, album) VALUES (@0, @1, @2, @3, @4)", cn)
        cmd.Parameters.Add("@0", filename)
        cmd.Parameters.Add("@1", title)
        cmd.Parameters.Add("@2", 0)
        cmd.Parameters.Add("@3", artist)
        cmd.Parameters.Add("@4", album)
        cmd.ExecuteNonQuery()

        elm = New mediaElement(filename, title, 0, artist, album)
        Me.items.Add(filename, elm)
        Return elm
    End Function

    ReadOnly Property connectionString() As String
        Get
            Return String.Format("DataSource=""{0}""; Password=""{1}""", LibraryFile, "library__d1@playr")
        End Get
    End Property


    ReadOnly Property LibraryFile As String
        Get
            Dim path() As String = { _
                Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), _
                "Playr", "Library", "library.sdf"}
            Return String.Join(IO.Path.DirectorySeparatorChar, path)
        End Get
    End Property

    Protected Overrides Sub Finalize()
        'Verbindung wieder schließen
        cn.Close()
        MyBase.Finalize()
    End Sub
End Class
